import { defineStore } from "pinia";
import { StudentObjectiveInfo } from "@/types";

interface CheckObjectiveStore {
  studentIds: string[];
  currentStudentId: string;
  student: StudentObjectiveInfo | null;
  currentImage: number;
  browsedImageIndexes: number[];
  answerMap: Record<string, { answer: string; isRight: boolean }>;
}

const useCheckObjectiveStore = defineStore<
  "checkObjective",
  CheckObjectiveStore
>("checkObjective", {
  state: () => ({
    studentIds: [],
    currentStudentId: "",
    student: null,
    currentImage: 0,
    browsedImageIndexes: [],
    answerMap: {},
  }),
  getters: {
    info(state: CheckObjectiveStore): CheckObjectiveStore {
      return { ...state };
    },
    studentCount(state: CheckObjectiveStore): number {
      return state.studentIds.length;
    },
    isLast(state: CheckObjectiveStore): boolean {
      return (
        state.studentIds.indexOf(state.currentStudentId) ===
        state.studentIds.length - 1
      );
    },
    isFirst(state: CheckObjectiveStore): boolean {
      return state.studentIds.indexOf(state.currentStudentId) === 0;
    },
    isMultiStudent(state: CheckObjectiveStore): boolean {
      return state.studentIds.length > 1;
    },
  },
  actions: {
    setInfo(partial: Partial<CheckObjectiveStore>) {
      this.$patch(partial);
    },
    resetInfo() {
      this.$reset();
    },
  },
  persist: {
    storage: sessionStorage,
  },
});

export default useCheckObjectiveStore;
