import { message } from "ant-design-vue";
import { studentSubjectiveConfirmData } from "@/api/checkPage";
import { useMarkStore } from "@/store";
import { computed, ref } from "vue";

export default function useTask(
  ids: string[],
  courseData: {
    examId: string;
    paperNumber: string;
  }
) {
  const markStore = useMarkStore();
  const studentIds = ref(ids);

  const currentStudentId = ref("");
  const currentIndex = computed(() =>
    studentIds.value.indexOf(currentStudentId.value)
  );
  const isFirst = computed(() => currentIndex.value === 0);
  const isLast = computed(
    () => currentIndex.value === studentIds.value.length - 1
  );
  const isMultiStudent = computed(() => studentIds.value.length > 1);

  async function getNextStudent() {
    if (isLast.value) {
      void message.warning("已经是最后一份！");
      return;
    }
    await updateTask(studentIds.value[currentIndex.value + 1]);
  }

  async function getPreviousStudent() {
    if (isFirst.value) {
      void message.warning("已经是第一份！");
      return;
    }
    await updateTask(studentIds.value[currentIndex.value - 1]);
  }

  async function updateTask(studentId) {
    const res = await studentSubjectiveConfirmData({
      studentId,
      ...courseData,
    });
    if (!res.data) {
      markStore.message = res.message || "数据错误";
      return;
    }
    res.data.taskId = res.data.studentId;
    const newTask = res.data;
    newTask.sheetUrls = newTask.sheetUrls || [];
    newTask.sliceUrls = [...newTask.sheetUrls];
    markStore.currentTask = newTask;
    currentStudentId.value = studentId;
  }

  return {
    getNextStudent,
    getPreviousStudent,
    studentIds,
    currentStudentId,
    currentIndex,
    isFirst,
    isLast,
    isMultiStudent,
  };
}
