import { Question } from "@/types";
import { useMarkStore } from "@/store";
import { watch, nextTick } from "vue";
import useTaskRejection from "./useTaskRejection";
import useQuestion from "./useQuestion";

/** chooseQuestion 当currentTask改变是，自动选择第一题 */
export default function useAutoChooseFirstQuestion() {
  const markStore = useMarkStore();
  const { showRejectedReason } = useTaskRejection();
  const { isDisabledQuestion } = useQuestion();

  // 监听currentTask的变化，自动选择第一题
  function watchCurrentTaskChooseQuestion() {
    watch(
      () => markStore.currentTask,
      () => {
        // 重置当前选择的quesiton和score
        markStore.currentQuestion = undefined;
        markStore.currentScore = undefined;

        const firstQuestion = markStore.currentTask?.questionList.find(
          (question) => !isDisabledQuestion(question)
        );
        if (firstQuestion) {
          setTimeout(() => {
            chooseQuestion(firstQuestion);
          }, 300);
        }
      },
      {
        immediate: true,
      }
    );
  }

  const scrollToQuestionOfBoard = async (question: Question) => {
    const node = document.querySelector(
      `#bq-${question.mainNumber}-${question.subNumber}`
    );
    const questionNode = document.querySelector(
      `#q-${question.mainNumber}-${question.subNumber}`
    );
    if (!questionNode) {
      // 非多媒体阅卷
      node && node.scrollIntoView({ block: "center", behavior: "smooth" });
      return;
    }
    // console.log(node);
    // node && node.scrollIntoView({ behavior: "smooth" });
    // if (node) node.scrollBy({ top: -50 });
    // setTimeout(() => {
    //   if (node) node.parentElement?.scrollTo({ top: 50, left: 0 });
    //   // node && node.scrollTop = 50//node.scrollIntoView({ behavior: "auto", block: "center" });
    //   if(node.)
    // }, 1500);
    async function checkIfEleMoving(ele: Element) {
      const { top: oldTop } = ele.getBoundingClientRect();
      await new Promise((res) => setTimeout(res, 200));
      // console.log(ele.getBoundingClientRect().top, oldTop);
      return ele.getBoundingClientRect().top - oldTop !== 0;
    }
    if (questionNode) {
      let isMoving = await checkIfEleMoving(questionNode);
      while (isMoving) {
        isMoving = await checkIfEleMoving(questionNode);
      }
      node && node.scrollIntoView({ block: "center", behavior: "smooth" });
    }
  };

  const scrollToQuestionArea = (question: Question) => {
    const area = question.picList && question.picList[0];
    if (!area) return;

    const areaNode = document.getElementById(
      `${area.i}_${area.x}_${area.y}_${area.w}_${area.h}`
    );
    if (!areaNode) return;
    areaNode.scrollIntoView({ block: "start", behavior: "smooth" });
  };

  function chooseQuestion(question: Question) {
    if (isDisabledQuestion(question)) return;

    markStore.currentQuestion = question;
    // FIXME: maybe should be an async function, temp fix for eslint

    void nextTick(() => {
      void scrollToQuestionOfBoard(question);
      scrollToQuestionArea(question);
      if (markStore.currentQuestion) {
        showRejectedReason(markStore.currentQuestion);
      }
    });
  }

  return {
    watchCurrentTaskChooseQuestion,
    chooseQuestion,
  };
}
