import { useMarkStore } from "@/store";
import { unref } from "vue";

export default function useFocusTracks() {
  const markStore = useMarkStore();

  let hovering = false;
  let timeoutId = -1;
  let removeTrackTimer: number | null = null;

  function addFocusTrack(
    groupNumber: number | undefined,
    mainNumber: number | undefined,
    subNumber: string | undefined,
    isMark?: boolean | undefined, // 是否是评卷，评卷时要考虑标记删除
    list?: any
  ) {
    hovering = true;

    timeoutId = setTimeout(() => {
      if (hovering) {
        _addFocusTrack(groupNumber, mainNumber, subNumber, isMark, list);
      }
    }, 200);
  }

  function _addFocusTrack(
    groupNumber: number | undefined,
    mainNumber: number | undefined,
    subNumber: string | undefined,
    isMark: boolean | undefined, // 是否是评卷，评卷时要考虑标记删除
    list?: any
  ) {
    markStore.focusTracks.splice(0);

    const listArr = unref(list);
    // console.log("listArr:", listArr);
    if (listArr) {
      const markerTrackList: any = list.map((q) => q.markerTrackList).flat();

      markerTrackList
        .filter((t) => {
          if (mainNumber) {
            return t.mainNumber === mainNumber && t.subNumber === subNumber;
          } else {
            return false;
          }
        })
        .forEach((t: any) => {
          // 回评时，如果没被删除
          const shouldAdd = isMark ? markerTrackList.includes(t) : true;
          if (shouldAdd) {
            markStore.focusTracks.push(t);
          }
        });
      return;
    }

    if (groupNumber) {
      markStore.currentTask?.questionList
        ?.filter((q) => q.groupNumber === groupNumber)
        ?.map((q) => q.markerTrackList)
        .flat()
        .forEach((t) => {
          // 回评时，如果没被删除
          const shouldAdd = isMark
            ? markStore.currentTask?.markResult.markerTrackList.includes(t)
            : true;
          if (shouldAdd) markStore.focusTracks.push(t);
        });
    } else {
      markStore.currentTask?.questionList
        ?.map((q) => q.markerTrackList)
        .flat()
        .filter((t) => {
          if (mainNumber) {
            return t.mainNumber === mainNumber && t.subNumber === subNumber;
          } else {
            return false;
          }
        })
        .forEach((t) => {
          // 回评时，如果没被删除
          const shouldAdd = isMark
            ? markStore.currentTask?.markResult.markerTrackList.includes(t)
            : true;
          if (shouldAdd) markStore.focusTracks.push(t);
        });
    }
  }

  function removeFocusTrack() {
    hovering = false;
    clearTimeout(timeoutId);
    removeTrackTimer && clearTimeout(removeTrackTimer);
    removeTrackTimer = setTimeout(() => {
      if (!hovering) {
        markStore.focusTracks.splice(0);
      }
    }, 200);
  }

  return { addFocusTrack, removeFocusTrack };
}
