import { clearMarkTask, getTask } from "@/api/markPage";
import { useMarkStore } from "@/store";
import useDraw from "./useDraw";
import useTaskQuestion from "./useTaskQuestion";

export default function useMarkTask() {
  const markStore = useMarkStore();
  const { processSliceUrls } = useDraw();
  const { getNextTaskQuestionId } = useTaskQuestion();

  let preDrawing = false;

  async function clearTask() {
    await clearMarkTask();
  }

  async function updateTask() {
    const nextQuestionId = getNextTaskQuestionId();
    const res = await getTask(markStore.setting.questionModel, nextQuestionId);
    if (res.data) {
      const newTask = res.data;
      newTask.sheetUrls = newTask.sheetUrls || [];
      try {
        preDrawing = true;
        newTask.sliceUrls = await processSliceUrls(newTask);
        newTask.sliceConfig = (newTask.sliceConfig || []).map((item) => {
          return {
            ...item,
            id: `${item.i}_${item.x}_${item.y}_${item.w}_${item.h}`,
          };
        });
      } finally {
        preDrawing = false;
      }

      markStore.tasks.push(newTask);
      if (!markStore.historyOpen) {
        if (markStore.currentTask?.studentId !== markStore.tasks[0].studentId)
          markStore.currentTask = markStore.tasks[0];
      }

      if (markStore.tasks.length > 1) {
        await new Promise((resolve) => setTimeout(resolve, 3000));
      }
    } else {
      markStore.message = res.message || "数据错误";
    }

    if (!res.data && markStore.message === "成功") {
      markStore.message = "当前无评卷任务";
    }
  }

  function nextTask() {
    // console.log("preDrawing", preDrawing);

    if (preDrawing) return;
    // console.log("markStore.tasks.length", markStore.tasks.length);
    // console.log(
    //   "markStore.setting.prefetchCount",
    //   markStore.setting.prefetchCount
    // );

    if (markStore.tasks.length < (markStore.setting.prefetchCount ?? 3)) {
      if (!markStore.historyOpen)
        return updateTask().catch((e) => console.log("定时获取任务出错", e));
    }
  }

  function removeBrokenTask() {
    markStore.currentTask = undefined;

    if (markStore.historyOpen) {
      // 避免无限加载
      markStore.message = "加载失败，请重新选择。";
    } else {
      markStore.tasks.shift();
    }
  }

  return {
    clearTask,
    updateTask,
    nextTask,
    removeBrokenTask,
  };
}
