import { Question } from "@/types";
import { useMarkStore } from "@/store";

export default function useQuestion() {
  const markStore = useMarkStore();

  // 是否处于仲裁阶段
  function isArbitrated(question: Question) {
    return ["WAIT_ARBITRATE", "ARBITRATED"].includes(question.status);
  }

  function getArbitratedStatusName(question: Question) {
    if (question.status === "WAIT_ARBITRATE") {
      return "待仲裁";
    } else if (question.status === "ARBITRATED") {
      return "已仲裁";
    }
    return "";
  }

  function isDisabledQuestion(question: Question) {
    return !question.selfMark || isArbitrated(question);
  }

  // 问题卷
  // 取消问题卷
  function cancelProblemQuestion() {
    if (!markStore.currentQuestion) return;

    markStore.currentQuestion.problem = false;
    markStore.currentQuestion.problemType = null;
    markStore.currentQuestion.problemRemark = "";
  }
  // 去标记问题卷
  function toMarkProblemQuestion() {
    if (!markStore.currentQuestion) return;

    markStore.problemModalVisible = true;
  }

  return {
    isDisabledQuestion,
    isArbitrated,
    getArbitratedStatusName,
    cancelProblemQuestion,
    toMarkProblemQuestion,
  };
}
