import { useMarkStore } from "@/store";
import { getStatus } from "@/api/markPage";
import { watch } from "vue";

export default function useStatus() {
  const markStore = useMarkStore();

  async function updateStatus() {
    const res = await getStatus(markStore.setting.questionModel);
    const statusData = (res.data || []).map((item) => {
      item.leftCount = item.leftCount <= 0 ? 0 : item.leftCount;
      return item;
    });
    markStore.setInfo({ status: statusData });

    // 单题阅的模式status可能有多条，需要依据评卷的试题来确定显示哪一条
    // 整卷阅的模式下只有一条，所以只取第一条数据
    if (!markStore.isSingelQuestionModel) {
      markStore.setInfo({ curStatus: statusData[0] });
      return;
    }

    // 单题阅的模式下，需要找到当前题目的状态, 并将当前题目的状态放到第一位
    const curMarkStatusIndex = statusData.findIndex(
      (item) => item.questionId === item.markedQuestionId
    );
    if (curMarkStatusIndex < 0) return;
    const statuss = statusData.slice(curMarkStatusIndex);
    statuss.push(...statusData.slice(0, curMarkStatusIndex));
    markStore.setInfo({ status: statuss });
  }

  // 单题阅的模式下才需要监听当前题目状态
  function listenStatusQuestion() {
    watch(
      () => markStore.currentQuestion,
      (val) => {
        if (!val || markStore.historyOpen) return;
        const { mainNumber, subNumber } = val;
        markStore.setInfo({
          curStatus: markStore.status.find(
            (item) =>
              item.mainNumber === mainNumber && item.subNumber === subNumber
          ),
        });
      }
    );
  }

  return {
    updateStatus,
    listenStatusQuestion,
  };
}
