import { useMarkStore } from "@/store";
import { getStatus } from "@/api/markPage";
import { MarkStatus } from "@/types";

export default function useTaskQuestion() {
  const markStore = useMarkStore();

  function getFirstUnfinishedQuestionStatus() {
    return markStore.status.find((item) => item.leftCount > 0);
  }

  // 获取下一个未评数量不为0的试题状态信息
  // function getNextQuestionStatus(questionId: string) {
  //   const questionIndex = markStore.status
  //     .filter((item) => item.questionId !== questionId)
  //     .findIndex((item) => item.leftCount > 0);
  //   if (questionIndex < 0) return;

  //   return markStore.status[questionIndex];
  // }

  function getNextTaskQuestionId() {
    if (!markStore.isSingelQuestionModel) return;

    const firstUnfinishedQuestion = getFirstUnfinishedQuestionStatus();
    if (!firstUnfinishedQuestion) return;

    let remainCount = markStore.tasks.length + 1;

    for (let i = 0; i < markStore.status.length; i++) {
      const curStatus = markStore.status[i];
      if (curStatus.leftCount <= 0) continue;
      remainCount = curStatus.leftCount - remainCount;
      if (remainCount >= 0) return curStatus.questionId;
    }

    return;
  }

  // 提交试题任务之后，更新试题状态信息
  function updateStatusQuestionCount(data: {
    questionId: string;
    statusData: Partial<MarkStatus>;
  }) {
    if (!data.questionId) return;

    const questionIndex = markStore.status.findIndex(
      (item) => item.questionId === data.questionId
    );
    if (questionIndex < 0) return;
    markStore.status[questionIndex] = Object.assign(
      markStore.status[questionIndex],
      data.statusData
    );
  }

  async function updateQuestionStatus(questionId: string) {
    const res = await getStatus(markStore.setting.questionModel, questionId);
    const statusData = (res.data || []).map((item) => {
      item.leftCount = item.leftCount <= 0 ? 0 : item.leftCount;
      return item;
    });
    updateStatusQuestionCount({
      questionId,
      statusData: statusData[0],
    });
  }

  return {
    getNextTaskQuestionId,
    updateStatusQuestionCount,
    updateQuestionStatus,
  };
}
