import type { AxiosResponse } from "axios";
interface SplitConfig {
  //  ${i}_${x}_${y}_${w}_${h}
  id: string;
  /** index of sheets */
  i: number;
  /** 覆盖区域的width */
  w: number;
  /** 覆盖区域的height */
  h: number;
  /** 从哪里开始覆盖 左上角为 (0, 0) */
  x: number;
  /** 从哪里开始覆盖 左上角为 (0, 0) */
  y: number;
}

interface MarkStatus {
  /** 个人评卷数量 */
  personCount: number;
  /** 总评卷数量 */
  markedCount: number;
  /** 总数量 */
  totalCount: number;
  // 待评数量
  leftCount: number;
  /** 问题卷数量 */
  problemCount: number;
  /** 待仲裁卷数量 */
  arbitrateCount: number;
  // 大题号
  mainNumber: number;
  // 小题号
  subNumber: number;
  // 题目Id
  questionId: string;
  // 当前正在评阅的题目Id
  markedQuestionId: string;
  /**成绩校验，批量校验时，已校验的学生id的集合 */
  markedCountStuIds?: any;
}

type SingleSheetConfig = SplitConfig;
export type PictureSlice = SplitConfig;
export interface MarkStore {
  setting: Setting;
  status: MarkStatus[];
  curStatus: MarkStatus;
  /** 保持数量为3(prefetchCount) */
  tasks: Array<Task>;
  /** 用来切换task，还有回看 */
  currentTask?: Task;
  currentTaskSliceImages?: Record<number, { width: number; height: number }>;
  sliceImagesWithTrackList: SliceImage[];
  currentQuestion?: Question;
  currentScore?: number;
  currentSpecialTag?: string;
  currentSpecialTagType?: SpecialTag.tagType;
  // 是否打开问题卷弹窗
  problemModalVisible: boolean;
  /** 是否打开选择评卷小题弹窗 */
  selectMarkingQuestionModalVisible: boolean;
  /** 是否打开回评侧边栏 */
  historyOpen: boolean;
  historyTasks: Array<Task>;
  /** 删除这些轨迹 */
  removeScoreTracks: Array<Track>;
  /** 聚焦这些tracks */
  focusTracks: Array<Track>;
  /** 缩略图设置滚动到宽度的百分比 */
  minimapScrollToX?: number;
  /** 缩略图设置滚动到高度的百分比 */
  minimapScrollToY?: number;
  /** 是否显示全卷 */
  allPaperModal: boolean;
  /** 是否显示原卷 */
  sheetViewModal: boolean;
  /** 是否全局遮盖 */
  globalMask: boolean;
  /** 任务渲染锁状态 */
  renderLock: boolean;
  /** 消息 */
  message: string | null;
}

export interface Setting {
  /** 扫描图片或者多媒体，多媒体只允许 common mode */
  examType: "SCAN_IMAGE" | "MULTI_MEDIA";
  /** 阅卷模式 TRACK | COMMON */
  mode: "TRACK" | "COMMON";
  /** 试题评阅方式 SINGLE：单题阅，MULTI：多题阅 */
  questionModel: "SINGLE" | "MULTI";
  /** 是否允许模式切换，true为不允许 */
  forceMode: boolean;
  /** 是否显示原图功能 */
  sheetView: boolean;
  /** 原图遮盖规则 */
  sheetConfig: Array<SingleSheetConfig>;
  /** 是否开启全零分 */
  enableAllZero: boolean;
  /** 图片服务地址 */
  fileServer: string;
  /** 评卷员姓名 */
  userName: string;
  /** 科目信息（试卷和答案功能）*/
  subject: Subject;
  /** 强制标记是否开启 */
  forceSpecialTag: boolean;
  /** 评卷小工具控制字段 */
  uiSetting: UISetting;
  /** 只显示试评名称 TRIAL("试评"), FORMAL("正评"), FINISH("结束"): 结束状态不会在评卷端出现 */
  statusValue: "TRIAL" | "FORMAL" | null;
  /** 问题卷类型 */
  problemTypes: Array<{ code: string; name: string }>;
  /** 推荐老师评卷的数量，到达这个数量提示老师 */
  topCount: number;
  /** 使用裁切整图时的裁切配置 [0,1]|[0,0.3,0.25,0.55], */
  splitConfig: Array<number>;
  /** 预加载任务数量 */
  prefetchCount: number;
  /** 评卷开始时间 */
  startTime: number;
  /** 评卷结束时间 */
  endTime: number;
  /** 是否是未选做类型 */
  selective: boolean;
  /** 是否自动跳转 */
  autoScroll: boolean;
  /** 是否允许裁切 */
  enableSplit: boolean;
  /** 评卷员页头是否展示客观分 */
  showObjectiveScore?: boolean;
  /** 可回评数量是否有限制数 */
  remarkCount?: any;
  /** 是否展示双评的轨迹 */
  doubleTrack?: boolean;
  /** 全卷复核页面是否要滑到页面底部才允许点击复核按钮 */
  inspectScroll?: boolean;
}

interface UrlData {
  url: string;
  paperType: string;
  filePathVo: string | null;
}
/** 科目信息（试卷和答案功能） */
interface RawSubject {
  /** 科目名称 */
  name: string;
  /** 科目编号 */
  code: string;
  /** 答案 url */
  answerUrl: UrlData[] | null;
  /** 试卷 url */
  paperUrl: UrlData[] | null;
}

/** 科目信息（试卷和答案功能）增加前端自定义 questions字段*/
interface Subject extends RawSubject {
  /** 前端自定义questions字段, 在多媒体阅卷模式下使用 */
  questions: Array<RichTextQuestion>;
}

// setting for admin page
export interface AdminPageSetting {
  /** 扫描图片或者多媒体，多媒体只允许 common mode */
  examType: "SCAN_IMAGE" | "MULTI_MEDIA";
  /** 图片服务地址 */
  fileServer: string;
  /** 管理员姓名 */
  userName: string;
  /** 科目信息 */
  subject: Subject;
  /** 使用裁切整图时的裁切配置 [0,1]|[0,0.3,0.25,0.55], */
  splitConfig: Array<number>;
  enableSplit: boolean;
  doubleTrack?: boolean;
  inspectScroll?: boolean;
}

export interface AdminPageSettingForImport extends AdminPageSetting {
  studentIds: number[];
  tagIds: number[];
  inspectCount: number;
  message: string;
}

export interface Group {
  /** 分组编号 */
  number: number;
  /** 分组试题 */
  groupQuestions: string;
  /** 分组title */
  title: string;
  /** 总评卷数量 */
  markedCount: number;
  /** 总数量 */
  totalCount: number;
}

export interface MarkArea {
  mainNumber: number;
  subNumber: number;
  // 1: 单选 2: 多选 3: 判断 4: 填空 5: 问答
  questionType: 1 | 2 | 3 | 4 | 5;
  splitConfig?: SplitConfig[];
}

interface RawTask {
  // 考试id
  examId: string;
  /** 学生ID */
  studentId: string;
  /** 任务编号 */
  secretNumber: string;
  /** 学生名称 */
  studentName: string;
  /** 学生编号 */
  studentCode: string;
  /** 课程名称 */
  courseName: string;
  /** 课程编号 */
  courseCode: string;
  /** 考试编号 */
  // examNumber: string;
  /** 试卷编号 */
  paperNumber: string;
  /** 试卷类型 */
  paperType: string;
  /** sliceUrls为空，则是多媒体阅卷，显示JSON */
  jsonUrl: string;
  /** 最高显示优先级，有sliceConfig就用sliceConfig，否则使用sheetConfig */
  sliceConfig: Array<SplitConfig>;
  /** 裁切图url */
  sliceUrls: Array<string>;
  // 试题列表
  questionList: Array<Question>;
  // 科组长特殊标记列表
  headerTagList?: any;
  /** 原图url */
  sheetUrls: Array<string>;
  /** 客观分 复核也用到 */
  objectiveScore: number;
  /** 评卷总分 */
  markerScore: number;
  /** 评卷时间 */
  markerTime: number;
  /** 复核有用 */
  inspectTime?: number;
  /** 是否回评 */
  previous: boolean;
  /** 是否是打回 */
  rejected: boolean;
  /** 打回原因 冒号拼接的 */
  message: string | null;
  /** 评卷员名称 */
  markerName: string | null;
  // 仲裁id
  arbitrateId?: string;
}

type ElementType =
  | "FILL_QUESTION"
  | "FILL_LINE"
  | "EXPLAIN"
  | "COMPOSITION"
  | "TOPIC_HEAD"
  | "CARD_HEAD";
interface CardBaseElement {
  id: string;
  type: ElementType;
  topicNo: number;
  startNumber: number;
  questionsCount: number;
}
interface CardElement extends CardBaseElement {
  parent: CardBaseElement;
}
export interface CardDataItem {
  exchange: {
    answer_area: Array<{
      main_number: number;
      sub_number: number | string;
      area: [number, number, number, number];
    }>;
    fill_area: Array<{
      field: "question" | "examNumber";
      index: number;
      single: boolean;
      horizontal: boolean;
      items: Array<{
        main_number: number;
        sub_number: number | string;
        options: [number, number, number, number][];
      }>;
    }>;
  };
  columns: Array<{
    elements: CardElement[];
  }>;
}

export interface Task extends RawTask {
  /** 评卷结果，在task第一次被访问时自动添加，watch currentTask */
  markResult: MarkResult;
  /** 前端自用，用于标记阅卷开始时间和计算spent */
  __markStartTime: number;
  // 是否是科组长任务
  // 主观题检查时，科组长特殊标记使用headerTagList
  __isMarkLeader: string;
  // 轨迹图中可能存在客观题填涂框数据，答案数据，题卡数据等
  answerMap?: Record<
    string,
    { answer: string; isRight: boolean; score: number; totalScore: number }
  >;
  recogDatas?: string[];
  cardData?: Array<CardDataItem>;
  markAreas?: MarkArea[];
  hasMarkArea?: boolean;
}

interface RawQuestion {
  /** 任务序号 */
  taskId: string;
  /** 试题号 */
  questionId: string;
  /** 大题号 */
  mainNumber: number;
  /** 小题号 */
  subNumber: number;
  /** 分数间隔 */
  intervalScore: number;
  /** 默认分数 */
  defaultScore: number;
  /** 题目名称 */
  title: string;
  /** 得分；null的值时是为打回时可以被评卷修改的；null也是从未评分过的情况，要通过rejected来判断 */
  score: number | null;
  markerScore: number | null;
  /** 限制最大分数 */
  maxScore: number;
  /** 限制最小分数 */
  minScore: number;
  picList: SplitConfig[];
  qeustionPicList: SplitConfig[] | null;
  // 1-单选，2-多选，3-判断，4-填空，5-问答
  questionType: 1 | 2 | 3 | 4 | 5;
  /** 未计分 */
  uncalculate: boolean;
  /** 是否自己评卷，false时为他人评卷，给分板置灰 */
  selfMark: boolean;
  // 任务状态： WAIT_ARBITRATE:待仲裁,PROBLEM:问题卷
  status: "WAIT_ARBITRATE" | "ARBITRATED" | "PROBLEM" | "WAITING" | "MARKED";
  /** 轨迹列表 */
  markerTrackList: Array<Track>;
  // 科组长轨迹
  headerTrackList?: Array<Track>;
  /** 无轨迹情况下评卷员打分信息 */
  markerList: null | Array<{
    // 是否是科组长
    header: boolean;
    loginName: string;
    score: number;
    userId: string;
    userName: string;
  }>;
  // 特殊标记列表
  markerTagList: Array<SpecialTag> | null;
  // 科组长特殊标记
  headerTagList: Array<SpecialTag> | null;
  // 是否打回
  rejected: boolean;
  // 打回原因
  rejectReason: string | null;
  // 打回前给分数据
  rejectScoreList: number[] | null;
  // 问题卷
  problem: boolean;
  // 问题卷类型
  problemType: string;
  // 问题卷备注
  problemRemark: string;
  questionName?: string;
}
export interface Question extends RawQuestion {
  /** question 在 task 里面的 index ，用来对应 scoreList 的 score */
  __index: number;
}

export interface ColorMap {
  [key: string]: string;
}
/** 轨迹数据 */
export interface Track {
  // 轨迹id: ${mainNumber}_${subNumber}_${randomCode(16)} */
  id: string;
  /** 大题号 */
  mainNumber: number;
  /** 小题号，当前api中只有number  */
  subNumber: number;
  /** 前端使用，暂时用不着，赋0 */
  number: number;
  /** 评分数 */
  score: number;
  /** 第几张图 */
  offsetIndex: number;
  /** 左上角为原点 */
  offsetX: number;
  offsetY: number;
  /** 是否此处未作答，未作答时，score默认是-0分 */
  unanswered: boolean;
  // 是否是复核员复核轨迹 headTrackList没有这个属性
  headerType: "ARBITRATED" | "MARKED" | null;
  userId?: string;
  userName?: string;
  color?: string;
  isByMultMark?: boolean;
}

/** 特殊标记数据 */
export interface SpecialTag {
  // 轨迹id: ${mainNumber}_${subNumber}_${randomCode(16)} */
  id: string;
  /** 大题号 */
  mainNumber: number;
  /** 小题号，当前api中只有number */
  subNumber: number;
  /** 第几张图 */
  offsetIndex: number;
  /** 左上角为原点(原图的原点)，及相对原图的位置比例 */
  offsetX: number;
  offsetY: number;
  /** 特殊标记的字符串，勾叉 */
  tagName: string;
  tagType: "TEXT" | "CIRCLE" | "RIGHT" | "WRONG" | "HALF_RIGTH" | "LINE";
  // 是否是复核员复核标记
  headerType: "ARBITRATED" | "MARKED" | null;
  // 分组号
  userId?: number;
  color?: string;
  isByMultMark?: boolean;
}

export interface UISetting {
  /** 试卷缩放比例 */
  "answer.paper.scale": number;
  /** 给分面板展示形态 */
  "score.board.collapse": boolean;
  /** 给分模式 "keyboard": 键盘模式 "mouse": 鼠标模式 */
  "normal.mode": "keyboard" | "mouse";
  /** 分数标记缩放比例 */
  "score.fontSize.scale": number;
  /** 弹窗显示试卷 */
  "paper.modal": boolean;
  /** 弹窗显示答案 */
  "answer.modal": boolean;
  /** 弹窗显示缩略图 */
  "minimap.modal": boolean;
  /** 弹窗显示特殊字符 */
  "specialTag.modal": boolean;
  /** 弹窗显示快捷键配置 */
  "shortCut.modal": boolean;
}

export interface MarkResultQuestion {
  /** 任务序号 */
  taskId: string;
  /** 试题号 */
  questionId: string;
  /** 大题号 */
  mainNumber: number;
  /** 小题号 */
  subNumber: number;
  // 轨迹 or 键盘
  markerScore: number | null;
  /** 轨迹列表 */
  markerTrackList: Array<Track>;
  /** 给分列表 */
  scoreList: Array<number | null>;
  /** 轨迹和键盘都需要 */
  markerTagList: Array<SpecialTag>;
  /** 问题卷 */
  problem: boolean;
  /** 问题卷类型 */
  problemType: string;
  /** 其他问题卷说明 */
  problemRemark: string;
  /** 当前task是否为学生未选做 */
  unselective: boolean;
}
export interface MarkResult {
  /** 考试id */
  examId: string;
  /** 试卷编号 */
  paperNumber: string;
  /** 学生id */
  studentId: number;
  /** 状态值 */
  statusValue: "TRIAL" | "FORMAL" | null;
  /** 毫秒单位 */
  spent: number;
  /** 试题列表 */
  questionList: Array<MarkResultQuestion>;
  /** 轨迹列表 */
  markerTrackList: Array<Track>;
  /** 特殊标记列表 */
  markerTagList: Array<SpecialTag>;
  /** 给分列表 */
  scoreList: Array<number | null>;
  /** 评卷员分数 */
  markerScore: number | null;
  // 当前正在评卷的题目ID
  markedQuestionId: string;
}

/** 前端自用，用来渲染裁切图 */
export interface SliceImage {
  // 裁切图区域ID
  sliceId: string;
  /** 当前是 ObjectURL , 因为 DataURL 性能太差 */
  url: string;
  indexInSliceUrls: number;
  markerTrackList: Array<Track>;
  tagList: Array<SpecialTag>;
  // originalImageWidth: number; // 为了兼容原图还原轨迹而添加的属性，当前CommonMarkBody用不到
  // originalImageHeight: number; // 为了兼容原图还原轨迹而添加的属性，当前CommonMarkBody用不到
  sliceImageWidth: number;
  sliceImageHeight: number;
  /** 裁切图在原图中的左上角的x偏移量 */
  dx: number;
  /** 裁切图在原图中的左上角的y偏移量 */
  dy: number;
  /** 在多个图片从高至低排列中累积的高度 */
  accumTopHeight: number;
  /** 当前裁切图有效宽度，大小不一的裁切图时有用。
   * 为了能让多张图统一比例的缩放，所以将所有的图的宽度设为一样了。 */
  effectiveWidth: number;
}

export type MarkHistoryOrderBy =
  | "marker_time"
  | "inspect_time"
  | "marker_score"
  | "seceret_number"
  | undefined;
export type MarkHistorySortField = "ASC" | "DESC" | undefined;

export interface HistoryQueryParams {
  /** 从1开始 */
  pageNumber?: number;
  pageSize?: number;
  order?: MarkHistoryOrderBy;
  sort?: MarkHistorySortField;
  secretNumber?: string | null;
  subjectCode?: string;
  groupNumber?: string;
  markerId?: string;
  markerScore?: string;
}

export interface GetHistory {
  (historyQuery: HistoryQueryParams): Promise<
    AxiosResponse<Task[]> | undefined
  >;
}

export interface CommonResponse {
  /** 请求是否成功 */
  success: boolean;
  /** 错误消息 */
  message: string;
}

/** 仲裁用：评卷明细 */
export interface MarkDetail {
  markerName: string;
  markerTime: number;
  totalScore: number;
  scoreList: string;
}

//#region  多媒体评卷
export interface RichTextQuestion {
  /** 题目的综合题号 1-2-4 */
  unionOrder: string;
  body: RichTextJSON;
  parentBody: RichTextJSON | null;
  answer: Array<RichTextJSON> | null;
  objective: boolean | null;
  options: Array<{ number: number; body: RichTextJSON }> | null;
  hideAnswer?: boolean;
}

export interface QuestionForRender extends Omit<RichTextQuestion, "answer"> {
  studentAnswer: RichTextQuestion["answer"];
  standardAnswer: RichTextQuestion["answer"];
  score: number | null;
  totalScore: number;
}

export interface RichTextJSON {
  sections: RichTextSectionJSON[];
}
export interface RichTextSectionJSON {
  blocks: RichTextBlockJSON[];
}
export interface RichTextBlockJSON {
  type: "text" | "image" | "audio" | "cloze";
  value: string;
  param: {
    underline: boolean;
    bold: boolean;
    italic: boolean;
    width: string;
    height: string;
  } | null;
}

export interface StudentAnswer {
  mainNumber: number;
  subNumber: number;
  subIndex: string;
  answer: Array<RichTextJSON> | null;
}

/** 云平台试卷格式 */
export type ECSPaperJSON = {
  mainNumber: number;
  subNumber: number;
  body: RichTextJSON;
  parentBody: RichTextJSON | null;
  answer: RichTextJSON;
}[];

/** 在线考试平台试卷格式 */
export type OExamPaperJSON = OExamPaperJSONQuestionList[];

interface OExamPaperJSONQuestionList {
  /** 大题号 */
  number: number;
  questions: OExamPaperJSONQuestion[];
}
interface OExamPaperJSONQuestion {
  number: number;
  body: RichTextJSON;
  answer: RichTextJSON[] | null;
  objective: boolean | null;
  options: Array<{ number: number; body: RichTextJSON }>;
  subQuestions: OExamPaperJSONQuestion[] | null;
  /** 1-单选，2-多选，3-判断，4-填空，5-问答，6-套题，7-听力，8-配对题 */
  structType: 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8;
}
//#endregion

export type StudentObjectiveInfo = {
  studentId: string;
  studentName: string;
  studentCode: string;
  campusName: string;
  courseCode: string;
  courseName: string;
  paperNumber: string;
  objectiveScore: number;
  subjectiveScore: number;
  upload: boolean;
  absent: boolean;
  paperType: string;
  sheetUrls: Array<{ index: number; url: string; recogData: string }>;
  answers: Array<{
    mainNumber: number;
    subNumber: number;
    answer: string;
    exist: boolean;
    questionType: string;
    standardAnswer: string;
    score: number;
    totalScore: number;
  }>;
  titles: { [index: number]: string };
  success: boolean;
};
export type StudentSubjectiveInfo = {
  studentId: string;
  studentName: string;
  studentCode: string;
  campusName: string;
  courseCode: string;
  courseName: string;
  paperNumber: string;
  objectiveScore: number;
  subjectiveScore: number;
  upload: boolean;
  absent: boolean;
  paperType: string;
  sheetUrls: Array<{ index: number; url: string }>;
  answers: Array<{
    mainNumber: number;
    subNumber: number;
    answer: string;
    exist: boolean;
    questionType: string;
  }>;
  titles: { [index: number]: string };
  success: boolean;
};

export interface StudentSubjectiveCheck {
  examId: string;
  paperNumber: string;
  studentId: string;
}

export interface PaperRecogData {
  page_index: number;
  question: Array<{
    index: number;
    fill_result: Array<{
      main_number: number;
      sub_number: number;
      single: number;
      fill_option: number[];
      suspect_flag: number;
      fill_position: string[];
      fill_size: number[];
    }>;
  }>;
}

export interface CardData {
  id: string;
  content: string;
}

// export interface SubjectiveQuestion extends RawQuestion {
//   picList: SplitConfig[];
//   // 1-单选，2-多选，3-判断，4-填空，5-问答
//   questionType: 1 | 2 | 3 | 4 | 5;
// }
export interface objectiveQuestion {
  mainNumber: number;
  subNumber: number;
  answer: string;
  exist: boolean;
  standardAnswer: string;
  score: number;
  totalScore: number;
  // 1-单选，2-多选，3-判断
  questionType: 1 | 2 | 3;
}
export interface StudentTrackTask {
  examId: string;
  studentId: string;
  secretNumber: string;
  studentName: string;
  studentCode: string;
  courseCode: string;
  courseName: string;
  paperNumber: string;
  paperType: string;
  objectiveScore: number;
  subjectiveScore: number;
  sheetUrls: Array<{ index: number; url: string; recogData: string }>;
  subjectiveQuestions: RawQuestion[];
  objectiveQuestions: objectiveQuestion[];
  cardContent: string;
}

// track-tag
export interface MarkDetailUserItem {
  userId: string;
  userName: string;
  prename: string;
  color: string;
  scores: Array<{ subNumber: number; score: number }>;
  score: number;
}
export type UserMapType = Record<string, MarkDetailUserItem>;

export interface QuestionArea {
  i: number;
  x: number;
  y: number;
  w: number;
  h: number;
  qStruct: string;
}
export interface MarkDetailItem {
  mainNumber: number;
  subNumber: number;
  isFillQuestion: boolean;
  score: number;
  maxScore: number;
  users: MarkDetailUserItem[];
  area: QuestionArea;
  style: Record<string, string>;
}

export interface AnswerTagItem {
  mainNumber: number;
  subNumber: number;
  answer: string;
  style: Record<string, string>;
}

export interface ObjectiveAnswerTagItem {
  id: string;
  mainNumber: number;
  subNumbers: string;
  score: number;
  totalScore: number;
  style: Record<string, string | number>;
}

export interface TrackSummaryItem {
  mainNumber: number;
  subNumber: number;
  score: number;
  markerName: string;
}
