// 打开cache后，会造成没有 vue devtools 时，canvas缓存错误，暂时不知道原因
// 通过回看的测试，打开回看，再关闭回看，稍等一会儿再打开回看，确实可以看到该缓存时缓存了，该丢弃时丢弃了

// 把store.currentTask当做 weakRef ，当它不存在时，就丢弃它所有的图片
// const weakedMapImages = new WeakMap<Object, Map<string, HTMLImageElement>>();

/**
 * 深拷贝
 * @param obj 需要拷贝的对象
 * @returns 拷贝后的对象
 */
export function deepCopy(obj: any) {
  return JSON.parse(JSON.stringify(obj));
}

/**
 * 异步获取图片
 * @param url 完整的图片路径
 * @returns Promise<HTMLImageElement>
 */
export async function loadImage(url: string): Promise<HTMLImageElement> {
  return new Promise((resolve, reject) => {
    const image = new Image();
    image.setAttribute("crossorigin", "anonymous");
    image.src = url;
    image.onload = () => {
      resolve(image);
    };
    image.onerror = () => {
      reject("图片载入错误");
    };
  });
}

/**
 * 获取随机code,默认获取16位
 * @param {Number} len 推荐8的倍数
 *
 */
export function randomCode(len = 16): string {
  if (len <= 0) return;
  const steps = Math.ceil(len / 8);
  const stepNums = [];
  for (let i = 0; i < steps; i++) {
    const ranNum = Math.random().toString(32).slice(-8);
    stepNums.push(ranNum);
  }

  return stepNums.join("");
}

/**
 * 解析url获取指定参数值
 * @param urlStr url
 * @param paramName 参数名
 * @returns 参数值
 */
export function parseHrefParam(
  urlStr: string,
  paramName: string = null
): string | object {
  if (!urlStr) return;
  const url = new URL(urlStr);

  const urlParams = new URLSearchParams(url.search);

  if (paramName) return urlParams.get(paramName);

  const params = {};
  for (const kv of urlParams.entries()) {
    params[kv[0]] = kv[1];
  }
  return params;
}

/**
 * 计算总数
 * @param {Array} dataList 需要统计的数组
 */
export function calcSum(dataList: number[]): number {
  if (!dataList.length) return 0;
  return dataList.reduce(function (total, item) {
    return total + item;
  }, 0);
}

export function toPrecision(num: number, fractionDigits = 1): number {
  if (fractionDigits < 0 || fractionDigits > 16) return num;

  const tnum = Math.pow(10, fractionDigits);
  return Math.round(num * tnum) / tnum;
}

export function calcSumPrecision(
  dataList: number[],
  fractionDigits = 1
): number {
  if (!dataList.length) return 0;
  const count = dataList.reduce(function (total, item) {
    return total + item;
  }, 0);
  return toPrecision(count, fractionDigits);
}

/** 获取数组最大数 */
export function maxNum(dataList: number[]): number {
  if (!dataList.length) return 0;
  return Math.max.apply(null, dataList);
}

/**
 * 百以下数字转中文汉字
 * @param {Number} num 大于0的100以下数字，其他数值直接转字符串
 *
 * @returns {String}
 */
export function numberToChinese(num) {
  if (num >= 100 || num <= 0) return num + "";
  const cnNums = "一二三四五六七八九十".split("");
  if (num <= 10) return cnNums[num - 1];

  return (num + "")
    .split("")
    .map((item) => item * 1)
    .map((item, index) => {
      if (index) {
        return !item ? "" : cnNums[item - 1];
      } else {
        return item === 1 ? "" : cnNums[item - 1];
      }
    })
    .join("十");
}

export function getTrackId(data: {
  mainNumber: number;
  subNumber: number;
}): string {
  return `${data.mainNumber}-${data.subNumber}-${randomCode()}`;
}
